@@ -16,7 +16,6 @@ import collections
 import copy
 import datetime
 
-from oslo_config import cfg
 from oslo_context import context as oslo_context
 from oslo_db.sqlalchemy import enginefacade
 
@@ -121,11 +120,9 @@ class ContextBase(oslo_context.RequestContext):
         context = copy.copy(self)
         context.is_admin = True
 
-        if 'admin' not in [x.lower() for x in context.roles]:
-            context.roles = context.roles + ["admin"]
-
-        if cfg.CONF.oslo_policy.enforce_new_defaults:
-            context.system_scope = 'all'
+        context.roles = list(
+            set(context.roles) | set(['admin', 'member', 'reader'])
+        )
 
         return context
 
@@ -182,13 +179,17 @@ class Context(ContextBaseWithSession):
 
 
 def get_admin_context():
+    # NOTE(slaweq): elevated() method will set is_admin=True but setting it
+    # explicity here will avoid checking in policy rules if is_admin should be
+    # set to True or not
     return Context(user_id=None,
                    tenant_id=None,
                    is_admin=True,
-                   overwrite=False)
+                   overwrite=False).elevated()
 
 
 def get_admin_context_without_session():
-    return ContextBase(user_id=None,
-                       tenant_id=None,
-                       is_admin=True)
+    # NOTE(slaweq): elevated() method will set is_admin=True but setting it
+    # explicity here will avoid checking in policy rules if is_admin should be
+    # set to True or not
+    return ContextBase(user_id=None, tenant_id=None, is_admin=True).elevated()
